@@ -19,7 +19,7 @@ from lib389._constants import DIRSRV_STATE_ONLINE
 from lib389._mapped_object_lint import DSLint, DSLints
 from lib389.utils import (
         ensure_bytes, ensure_str, ensure_int, ensure_list_bytes, ensure_list_str,
-        ensure_list_int, display_log_value, display_log_data
+        ensure_list_int, display_log_value, display_log_data, is_a_dn, normalizeDN
         )
 
 # This function filter and term generation provided thanks to
@@ -73,7 +73,7 @@ def _ldap_op_s(inst, f, fname, *args, **kwargs):
     try:
         return f(*args, **kwargs)
     except ldap.LDAPError as e:
-        new_desc = f"{fname}({args},{kwargs}) on instance {inst.serverid}";
+        new_desc = f"{fname}({args},{kwargs}) on instance {inst.serverid}"
         if len(e.args) >= 1:
             e.args[0]['ldap_request'] = new_desc
             logging.getLogger().debug(f"args={e.args}")
@@ -145,12 +145,31 @@ class DSLdapObject(DSLogging, DSLint):
         val = self._dn
         if self._rdn_attribute:
             # What if the rdn is multi value and we don't get the primary .... ARGHHH
-            val = self.get_attr_val(self._rdn_attribute)
+            # TODO: https://github.com/389ds/389-ds-base/issues/3329
+            val = self.get_rdn_from_dn()
         return ensure_str(val)
 
+    def get_rdn_from_dn(self, dn=None):
+        """Extract the RDN value from a DN.
+
+        :param dn: DN to extract RDN from, defaults to this object's DN
+        :type dn: str
+        :returns: The RDN value
+        :rtype: str
+        """
+        if dn is None:
+            dn = self._dn
+
+        rdn_components = ldap.dn.str2dn(dn)[0]
+        rdn_value = rdn_components[0][1]
+        return rdn_value
+
     def __str__(self):
         return self.__unicode__()
 
+    def __repr__(self):
+        return f'{type(self)}(instance_name="{self._instance.serverid}", dn="{self.__unicode__()}")'
+
     def _unsafe_raw_entry(self):
         """Get an Entry object
 
@@ -261,7 +280,8 @@ class DSLdapObject(DSLogging, DSLint):
         """
 
         # How can we be sure this returns the primary one?
-        return ensure_str(self.get_attr_val(self._rdn_attribute))
+        # TODO: https://github.com/389ds/389-ds-base/issues/3329
+        return ensure_str(self.get_rdn_from_dn())
 
     def get_basedn(self):
         """Get the suffix this entry belongs to
@@ -292,15 +312,28 @@ class DSLdapObject(DSLogging, DSLint):
         _search_ext_s(self._instance,self._dn, ldap.SCOPE_BASE, self._object_filter, attrlist=[attr, ],
                                         serverctrls=self._server_controls, clientctrls=self._client_controls,
                                         escapehatch='i am sure')[0]
-        values = self.get_attr_vals_bytes(attr)
+        values = self.get_attr_vals_utf8(attr)
         self._log.debug("%s contains %s" % (self._dn, values))
 
         if value is None:
             # We are just checking if SOMETHING is present ....
             return len(values) > 0
+
+        # Otherwise, we are checking a specific value
+        if is_a_dn(value):
+            normalized_value = normalizeDN(value)
         else:
-            # Check if a value really does exist.
-            return ensure_bytes(value).lower() in [x.lower() for x in values]
+            normalized_value = ensure_bytes(value).lower()
+
+        # Normalize each returned value depending on whether it is a DN
+        normalized_values = []
+        for v in values:
+            if is_a_dn(v):
+                normalized_values.append(normalizeDN(v))
+            else:
+                normalized_values.append(ensure_bytes(v.lower()))
+
+        return normalized_value in normalized_values
 
     def add(self, key, value):
         """Add an attribute with a value
@@ -318,25 +351,31 @@ class DSLdapObject(DSLogging, DSLint):
         This is useful for configuration changes that require
         atomic operation, and ease of use.
 
-        An example of usage is add_many((key, value), (key, value))
+        An example of usage is add_many((key, value), (key, [value1, value2]))
 
         No wrapping list is needed for the arguments.
 
-        :param *args: tuples of key,value to replace.
-        :type *args: (str, str)
+        :param *args: tuples of key,value to add. Value can be a single value
+                    or a collection (list, tuple, set) of values.
+        :type *args: (str, str) or (str, list/tuple/set)
         """
-
         mods = []
         for arg in args:
-            if isinstance(arg[1], list) or isinstance(arg[1], tuple):
-                value = ensure_list_bytes(arg[1])
+            key, value = arg
+            if isinstance(value, (list, tuple, set)):
+                value = ensure_list_bytes(list(value))
             else:
-                value = [ensure_bytes(arg[1])]
-            mods.append((ldap.MOD_ADD, ensure_str(arg[0]), value))
-        return _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls,
-                                            clientctrls=self._client_controls, escapehatch='i am sure')
+                value = [ensure_bytes(value)]
+
+            mods.append((ldap.MOD_ADD, ensure_str(key), value))
+
+        return _modify_ext_s(self._instance,
+                            self._dn,
+                            mods,
+                            serverctrls=self._server_controls,
+                            clientctrls=self._client_controls,
+                            escapehatch='i am sure')
 
-    # Basically what it means;
     def replace(self, key, value):
         """Replace an attribute with a value
 
@@ -352,23 +391,30 @@ class DSLdapObject(DSLogging, DSLint):
         This is useful for configuration changes that require
         atomic operation, and ease of use.
 
-        An example of usage is replace_many((key, value), (key, value))
+        An example of usage is replace_many((key, value), (key, [value1, value2]))
 
         No wrapping list is needed for the arguments.
 
-        :param *args: tuples of key,value to replace.
-        :type *args: (str, str)
+        :param *args: tuples of key,value to replace. Value can be a single value
+                    or a collection (list, tuple, set) of values.
+        :type *args: (str, str) or (str, list/tuple/set)
         """
-
         mods = []
         for arg in args:
-            if isinstance(arg[1], list) or isinstance(arg[1], tuple):
-                value = ensure_list_bytes(arg[1])
+            key, value = arg
+            if isinstance(value, (list, tuple, set)):
+                value = ensure_list_bytes(list(value))
             else:
-                value = [ensure_bytes(arg[1])]
-            mods.append((ldap.MOD_REPLACE, ensure_str(arg[0]), value))
-        return _modify_ext_s(self._instance,self._dn, mods, serverctrls=self._server_controls,
-                                           clientctrls=self._client_controls, escapehatch='i am sure')
+                value = [ensure_bytes(value)]
+
+            mods.append((ldap.MOD_REPLACE, ensure_str(key), value))
+
+        return _modify_ext_s(self._instance,
+                            self._dn,
+                            mods,
+                            serverctrls=self._server_controls,
+                            clientctrls=self._client_controls,
+                            escapehatch='i am sure')
 
     # This needs to work on key + val, and key
     def remove(self, key, value):
@@ -511,9 +557,10 @@ class DSLdapObject(DSLogging, DSLint):
         elif value is not None:
             value = [ensure_bytes(value)]
 
-        return _modify_ext_s(self._instance,self._dn, [(action, key, value)],
-                                           serverctrls=self._server_controls, clientctrls=self._client_controls,
-                                           escapehatch='i am sure')
+        return _modify_ext_s(self._instance, self._dn, [(action, key, value)],
+                             serverctrls=self._server_controls,
+                             clientctrls=self._client_controls,
+                             escapehatch='i am sure')
 
     def apply_mods(self, mods):
         """Perform modification operation using several mods at once
@@ -936,7 +983,7 @@ class DSLdapObject(DSLogging, DSLint):
                 if properties.get(attr, None) is None:
                     # Put RDN to properties
                     if attr == self._rdn_attribute and rdn is not None:
-                        properties[self._rdn_attribute] = ldap.dn.str2dn(rdn)[0][0][1]
+                        properties[self._rdn_attribute] = self.get_rdn_from_dn(rdn)
                     else:
                         raise ldap.UNWILLING_TO_PERFORM('Attribute %s must not be None' % attr)
 
@@ -1121,17 +1168,22 @@ class DSLdapObjects(DSLogging, DSLints):
         # functions with very little work on the behalf of the overloader
         return self._childobject(instance=self._instance, dn=dn)
 
-    def list(self, paged_search=None, paged_critical=True):
-        """Get a list of children entries (DSLdapObject, Replica, etc.) using a base DN
-        and objectClasses of our object (DSLdapObjects, Replicas, etc.)
+    def list(self, paged_search=None, paged_critical=True, full_dn=False):
+        """Get a list of children entries (DSLdapObject, Replica, etc.) using
+        a base DN and objectClasses of our object (DSLdapObjects, Replicas,
+        etc.)
 
-        :param paged_search: None for no paged search, or an int of page size to use.
+        :param paged_search: None for no paged search, or an int of page size
+                             to use.
+        :param paged_critical: pages search is critical
+        :param full_dn: Return a list of DN's instead of objects
         :returns: A list of children entries
         """
 
         # Filter based on the objectclasses and the basedn
         insts = None
-        # This will yield and & filter for objectClass with as many terms as needed.
+        # This will yield an & filter for objectClass with as many terms as
+        # needed.
         filterstr = self._get_objectclass_filter()
         self._log.debug('list filter = %s' % filterstr)
 
@@ -1141,7 +1193,9 @@ class DSLdapObjects(DSLogging, DSLints):
             results = []
             pages = 0
             pctrls = []
-            req_pr_ctrl = SimplePagedResultsControl(paged_critical, size=paged_search, cookie='')
+            req_pr_ctrl = SimplePagedResultsControl(paged_critical,
+                                                    size=paged_search,
+                                                    cookie='')
             if self._server_controls is not None:
                 controls = [req_pr_ctrl] + self._server_controls
             else:
@@ -1157,12 +1211,12 @@ class DSLdapObjects(DSLogging, DSLints):
                         escapehatch='i am sure'
                     )
                 self._log.info('Getting page %d' % (pages,))
-                rtype, rdata, rmsgid, rctrls = self._instance.result3(msgid, escapehatch='i am sure')
+                rtype, rdata, rmsgid, rctrls = self._instance.result3(msgid,
+                                                                      escapehatch='i am sure')
                 results.extend(rdata)
                 pages += 1
                 self._log.debug("%s" % rctrls)
-                pctrls = [ c for c in rctrls
-                    if c.controlType == SimplePagedResultsControl.controlType]
+                pctrls = [c for c in rctrls if c.controlType == SimplePagedResultsControl.controlType]
                 if pctrls and pctrls[0].cookie:
                     req_pr_ctrl.cookie = pctrls[0].cookie
                     if self._server_controls is not None:
@@ -1171,26 +1225,32 @@ class DSLdapObjects(DSLogging, DSLints):
                         controls = [req_pr_ctrl]
                 else:
                     break
-                #End while
+                # End while
             # Result3 doesn't map through Entry, so we have to do it manually.
             results = [Entry(r) for r in results]
-            insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+            if full_dn:
+                insts = [r.dn for r in results]
+            else:
+                insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
             # End paged search
         else:
             # If not paged
             try:
                 results = _search_ext_s(self._instance,
-                    base=self._basedn,
-                    scope=self._scope,
-                    filterstr=filterstr,
-                    attrlist=self._list_attrlist,
-                    serverctrls=self._server_controls, clientctrls=self._client_controls,
-                    escapehatch='i am sure'
-                )
-                # def __init__(self, instance, dn=None):
-                insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
+                                        base=self._basedn,
+                                        scope=self._scope,
+                                        filterstr=filterstr,
+                                        attrlist=self._list_attrlist,
+                                        serverctrls=self._server_controls,
+                                        clientctrls=self._client_controls,
+                                        escapehatch='i am sure')
+                if full_dn:
+                    insts = [r.dn for r in results]
+                else:
+                    insts = [self._entry_to_instance(dn=r.dn, entry=r) for r in results]
             except ldap.NO_SUCH_OBJECT:
-                # There are no objects to select from, se we return an empty array
+                # There are no objects to select from, se we return an empty
+                # array
                 insts = []
         return insts
 
@@ -1240,7 +1300,10 @@ class DSLdapObjects(DSLogging, DSLints):
         if len(results) == 0:
             raise ldap.NO_SUCH_OBJECT(f"No object exists given the filter criteria: {criteria} {search_filter}")
         if len(results) > 1:
-            raise ldap.UNWILLING_TO_PERFORM(f"Too many objects matched selection criteria: {criteria} {search_filter}")
+            entry_dn = [e.dn for e in results]
+            entry_dns_pretty = '\n    '.join(entry_dn)
+            raise ldap.UNWILLING_TO_PERFORM(f"Too many objects matched selection criteria: {criteria} {search_filter}"
+                                            f" - Please use 'get-by-dn' to specify which entry to get:\n    {entry_dns_pretty}")
         if json:
             return self._entry_to_instance(results[0].dn, results[0]).get_all_attrs_json()
         else:
